@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2003 Zope Corporation and Contributors.
+# Copyright (c) 2003 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -14,29 +14,28 @@
 """Mail Delivery Tests
 
 Simple implementation of the MailDelivery, Mailers and MailEvents.
-
-$Id: test_delivery.py 110390 2010-04-01 12:33:34Z mgedmin $
 """
-
 import smtplib
-from unittest import TestCase, TestSuite, makeSuite, main
-import doctest
+from unittest import TestCase, TestSuite, makeSuite
 
 import transaction
-from zope.interface import implements
+from zope.interface import implementer
 from zope.interface.verify import verifyObject
 from zope.sendmail.interfaces import IMailer
 
 
+@implementer(IMailer)
 class MailerStub(object):
 
-    implements(IMailer)
     def __init__(self, *args, **kw):
         self.sent_messages = []
 
     def send(self, fromaddr, toaddrs, message):
         self.sent_messages.append((fromaddr, toaddrs, message))
 
+    abort = None
+    vote = None
+
 
 class TestMailDataManager(TestCase):
 
@@ -48,46 +47,49 @@ class TestMailDataManager(TestCase):
         self.assertEqual(manager.callable, object)
         self.assertEqual(manager.args, (1, 2))
 
+    def test_successful_commit(self):
+        #Regression test for http://www.zope.org/Collectors/Zope3-dev/590
+        from zope.sendmail.delivery import MailDataManager
 
-def print_success(*args):
-    print "message successfully sent, args: %s" % (args, )
+        _success = []
+        def _on_success(*args):
+            _success.append(args)
+        _abort = []
+        def _on_abort(*args):
+            _abort.append(args)
 
-def print_abort():
-    print "message aborted"
+        manager = MailDataManager(_on_success, ('foo', 'bar'),
+                                  onAbort=_on_abort)
+        xact = object()
+        manager.tpc_begin(xact)
+        manager.commit(xact)
+        manager.tpc_vote(xact)
+        manager.tpc_finish(xact)
+        self.assertEqual(_success, [('foo', 'bar')])
+        self.assertEqual(_abort, [])
 
+    def test_unsuccessful_commit(self):
+        #Regression test for http://www.zope.org/Collectors/Zope3-dev/590
+        from zope.sendmail.delivery import MailDataManager
 
-def doctest_successful_commit():
-    """Regression test for http://www.zope.org/Collectors/Zope3-dev/590
+        _success = []
+        _abort = []
 
-    Let's do a full two-phase commit.
-
-        >>> from zope.sendmail.delivery import MailDataManager
-        >>> manager = MailDataManager(print_success, ('foo', 'bar'),
-        ...                           onAbort=print_abort)
-        >>> transaction = object()
-        >>> manager.tpc_begin(transaction)
-        >>> manager.commit(transaction)
-        >>> manager.tpc_vote(transaction)
-        >>> manager.tpc_finish(transaction)
-        message successfully sent, args: ('foo', 'bar')
-
-    """
-
-
-def doctest_unsuccessful_commit():
-    """Regression test for http://www.zope.org/Collectors/Zope3-dev/590
-
-    Let's start a two-phase commit, then abort it.
-
-        >>> from zope.sendmail.delivery import MailDataManager
-        >>> manager = MailDataManager(print_success, onAbort=print_abort)
-        >>> manager.tpc_begin(transaction)
-        >>> manager.commit(transaction)
-        >>> manager.tpc_vote(transaction)
-        >>> manager.tpc_abort(transaction)
-        message aborted
-
-    """
+        _success = []
+        def _on_success(*args):
+            _success.append(args)
+        _abort = []
+        def _on_abort(*args):
+            _abort.append(args)
+        manager = MailDataManager(_on_success, ('foo', 'bar'),
+                                  onAbort=_on_abort)
+        xact = object()
+        manager.tpc_begin(xact)
+        manager.commit(xact)
+        manager.tpc_vote(xact)
+        manager.tpc_abort(xact)
+        self.assertEqual(_success, [])
+        self.assertEqual(_abort, [()])
 
 
 class TestDirectMailDelivery(TestCase):
@@ -116,30 +118,86 @@ class TestDirectMailDelivery(TestCase):
                        'This is just an example\n')
 
         msgid = delivery.send(fromaddr, toaddrs, opt_headers + message)
-        self.assertEquals(msgid, '20030519.1234@example.org')
-        self.assertEquals(mailer.sent_messages, [])
+        self.assertEqual(msgid, '20030519.1234@example.org')
+        self.assertEqual(mailer.sent_messages, [])
         transaction.commit()
-        self.assertEquals(mailer.sent_messages,
+        self.assertEqual(mailer.sent_messages,
                           [(fromaddr, toaddrs, opt_headers + message)])
 
         mailer.sent_messages = []
         msgid = delivery.send(fromaddr, toaddrs, message)
-        self.assert_('@' in msgid)
-        self.assertEquals(mailer.sent_messages, [])
+        self.assertTrue('@' in msgid)
+        self.assertEqual(mailer.sent_messages, [])
         transaction.commit()
-        self.assertEquals(len(mailer.sent_messages), 1)
-        self.assertEquals(mailer.sent_messages[0][0], fromaddr)
-        self.assertEquals(mailer.sent_messages[0][1], toaddrs)
-        self.assert_(mailer.sent_messages[0][2].endswith(message))
+        self.assertEqual(len(mailer.sent_messages), 1)
+        self.assertEqual(mailer.sent_messages[0][0], fromaddr)
+        self.assertEqual(mailer.sent_messages[0][1], toaddrs)
+        self.assertTrue(mailer.sent_messages[0][2].endswith(message))
         new_headers = mailer.sent_messages[0][2][:-len(message)]
-        self.assert_(new_headers.find('Message-Id: <%s>' % msgid) != -1)
+        self.assertTrue(new_headers.find('Message-Id: <%s>' % msgid) != -1)
 
         mailer.sent_messages = []
         msgid = delivery.send(fromaddr, toaddrs, opt_headers + message)
-        self.assertEquals(mailer.sent_messages, [])
+        self.assertEqual(mailer.sent_messages, [])
         transaction.abort()
-        self.assertEquals(mailer.sent_messages, [])
+        self.assertEqual(mailer.sent_messages, [])
 
+    def testBrokenMailerErrorsAreEaten(self):
+        from zope.testing.loggingsupport import InstalledHandler
+        from zope.sendmail.delivery import DirectMailDelivery
+        mailer = BrokenMailerStub()
+        delivery = DirectMailDelivery(mailer)
+        fromaddr = 'Jim <jim@example.com'
+        toaddrs = ('Guido <guido@example.com>',
+                   'Steve <steve@examplecom>')
+        opt_headers = ('From: Jim <jim@example.org>\n'
+                       'To: some-zope-coders:;\n'
+                       'Date: Mon, 19 May 2003 10:17:36 -0400\n'
+                       'Message-Id: <20030519.1234@example.org>\n')
+        message =     ('Subject: example\n'
+                       '\n'
+                       'This is just an example\n')
+
+        msgid = delivery.send(fromaddr, toaddrs, opt_headers + message)
+        log_handler = InstalledHandler('MailDataManager')
+        try:
+            transaction.commit()
+        finally:
+            # Clean up after ourselves
+            log_handler.uninstall()
+            transaction.abort()
+
+    def testRefusingMailerDiesInVote(self):
+        from zope.sendmail.delivery import DirectMailDelivery
+        mailer = RefusingMailerStub()
+        delivery = DirectMailDelivery(mailer)
+        fromaddr = 'Jim <jim@example.com'
+        toaddrs = ('Guido <guido@example.com>',
+                   'Steve <steve@examplecom>')
+        opt_headers = ('From: Jim <jim@example.org>\n'
+                       'To: some-zope-coders:;\n'
+                       'Date: Mon, 19 May 2003 10:17:36 -0400\n'
+                       'Message-Id: <20030519.1234@example.org>\n')
+        message =     ('Subject: example\n'
+                       '\n'
+                       'This is just an example\n')
+
+        msgid = delivery.send(fromaddr, toaddrs, opt_headers + message)
+        try:
+            try:
+                transaction.commit()
+            except:
+                if transaction.get()._voted:
+                    # We voted for commit then failed, reraise
+                    raise
+                else:
+                    # We vetoed a commit, that's good.
+                    pass
+            else:
+                self.fail("Did not raise an exception in vote")
+        finally:
+            # Clean up after ourselves
+            transaction.abort()
 
 class MaildirWriterStub(object):
 
@@ -213,19 +271,36 @@ class BizzarreMailError(IOError):
     pass
 
 
+@implementer(IMailer)
 class BrokenMailerStub(object):
 
-    implements(IMailer)
     def __init__(self, *args, **kw):
         pass
 
     def send(self, fromaddr, toaddrs, message):
         raise BizzarreMailError("bad things happened while sending mail")
 
+    vote = None
+    abort = None
 
+
+@implementer(IMailer)
+class RefusingMailerStub(object):
+
+    def __init__(self, *args, **kw):
+        pass
+
+    def vote(self, fromaddr, toaddrs, message):
+        raise BizzarreMailError("bad things happened while sending mail")
+
+    def send(self, fromaddr, toaddrs, message):
+        return
+
+    abort = None
+
+@implementer(IMailer)
 class SMTPResponseExceptionMailerStub(object):
 
-    implements(IMailer)
     def __init__(self, code):
         self.code = code
 
@@ -270,35 +345,35 @@ class TestQueuedMailDelivery(TestCase):
                        'This is just an example\n')
 
         msgid = delivery.send(fromaddr, toaddrs, opt_headers + message)
-        self.assertEquals(msgid, '20030519.1234@example.org')
-        self.assertEquals(MaildirWriterStub.commited_messages, [])
-        self.assertEquals(MaildirWriterStub.aborted_messages, [])
+        self.assertEqual(msgid, '20030519.1234@example.org')
+        self.assertEqual(MaildirWriterStub.commited_messages, [])
+        self.assertEqual(MaildirWriterStub.aborted_messages, [])
         transaction.commit()
-        self.assertEquals(MaildirWriterStub.commited_messages,
+        self.assertEqual(MaildirWriterStub.commited_messages,
                           [zope_headers + opt_headers + message])
-        self.assertEquals(MaildirWriterStub.aborted_messages, [])
+        self.assertEqual(MaildirWriterStub.aborted_messages, [])
 
         MaildirWriterStub.commited_messages = []
         msgid = delivery.send(fromaddr, toaddrs, message)
-        self.assert_('@' in msgid)
-        self.assertEquals(MaildirWriterStub.commited_messages, [])
-        self.assertEquals(MaildirWriterStub.aborted_messages, [])
+        self.assertTrue('@' in msgid)
+        self.assertEqual(MaildirWriterStub.commited_messages, [])
+        self.assertEqual(MaildirWriterStub.aborted_messages, [])
         transaction.commit()
-        self.assertEquals(len(MaildirWriterStub.commited_messages), 1)
-        self.assert_(MaildirWriterStub.commited_messages[0].endswith(message))
+        self.assertEqual(len(MaildirWriterStub.commited_messages), 1)
+        self.assertTrue(MaildirWriterStub.commited_messages[0].endswith(message))
         new_headers = MaildirWriterStub.commited_messages[0][:-len(message)]
-        self.assert_(new_headers.find('Message-Id: <%s>' % msgid) != -1)
-        self.assert_(new_headers.find('X-Zope-From: %s' % fromaddr) != 1)
-        self.assert_(new_headers.find('X-Zope-To: %s' % ", ".join(toaddrs)) != 1)
-        self.assertEquals(MaildirWriterStub.aborted_messages, [])
+        self.assertTrue(new_headers.find('Message-Id: <%s>' % msgid) != -1)
+        self.assertTrue(new_headers.find('X-Zope-From: %s' % fromaddr) != 1)
+        self.assertTrue(new_headers.find('X-Zope-To: %s' % ", ".join(toaddrs)) != 1)
+        self.assertEqual(MaildirWriterStub.aborted_messages, [])
 
         MaildirWriterStub.commited_messages = []
         msgid = delivery.send(fromaddr, toaddrs, opt_headers + message)
-        self.assertEquals(MaildirWriterStub.commited_messages, [])
-        self.assertEquals(MaildirWriterStub.aborted_messages, [])
+        self.assertEqual(MaildirWriterStub.commited_messages, [])
+        self.assertEqual(MaildirWriterStub.aborted_messages, [])
         transaction.abort()
-        self.assertEquals(MaildirWriterStub.commited_messages, [])
-        self.assertEquals(len(MaildirWriterStub.aborted_messages), 1)
+        self.assertEqual(MaildirWriterStub.commited_messages, [])
+        self.assertEqual(len(MaildirWriterStub.aborted_messages), 1)
 
 
 def test_suite():
@@ -306,8 +381,4 @@ def test_suite():
         makeSuite(TestMailDataManager),
         makeSuite(TestDirectMailDelivery),
         makeSuite(TestQueuedMailDelivery),
-        doctest.DocTestSuite(),
         ))
-
-if __name__ == '__main__':
-    main()
